####################
# Zope Army Knife  #
# (c) 2003 NAUMEN  #
####################
"""
Revision information:
$Id: Watch.py,v 1.1 2003/08/27 11:20:22 kvas Exp $

ZAK class
"""
__version__ = '$Revision: 1.1 $'[11:-2]

from __future__ import nested_scopes
from threading import Lock
from zLOG import LOG, WARNING
from traceback import extract_stack, format_list
from threading import currentThread
from time import strftime

class Watch:
    """Method invocation watch
    
    Intercepts calls to a specific method and logs them
    """
    __roles__ = None
    __allow_access_to_unprotected_subobjects__ = 1

    def __init__(self, method_path):
        """creates new watch and attaches it to specified class"""
        self.__method_path = method_path
        self.__call_records = []
        self.__attached = 0
        self.attach()

    def clear(self):
        """Forget all call records"""
        self.__call_records = []

    def addCR(self, cr):
        """Log method call"""
        self.__call_records.append(cr)

    def getCR(self, index):
        """Return method call record"""
        return self.__call_records[int(index)]

    def makeWrapper(self, wrappee, docstring=None):
        """Create a wrapper around wrappee"""
        code = compile("""def wrapper(*args, **kwargs):
            cr = CallRecord(extract_stack(), args, kwargs, currentThread())
            ret_val = wrappee(*args, **kwargs)
            cr.storeRetVal(ret_val)
            watch.addCR(cr)
            return ret_val\n""", 'makeWrapper_virtual', 'exec')
        g = { 'CallRecord': CallRecord,
              'extract_stack': extract_stack,
              'currentThread': currentThread }
        eval(code, g)
        wrapper = g['wrapper']
        wrapper.func_globals['watch'] = self
        wrapper.func_globals['wrappee'] = wrappee
        if docstring is not None:
            wrapper.__doc__ = docstring
        return wrapper
        
    def attach(self):
        """attach self to specified class"""
        if self.__attached: return

        t = self.__method_path.split('.')
        try:                               # try to import as a module function
            module = '.'.join(t[:-1])
            func_name = t[-1]
            module = __import__(module, globals(), locals(), ['zoo'])
            func = getattr(module, func_name)
            setattr(module, func_name, self.makeWrapper(func))
            self.__type = 'module'
            self.__func_name = func_name
            self.__module = module
        except:                             # try as a class method
            module = '.'.join(t[:-2])
            class_name = t[-2]
            method_name = t[-1]
            module = __import__(module, globals(), locals(), ['zoo'])
            klass = getattr(module, class_name)
            method = getattr(klass, method_name).im_func
            setattr(klass, method_name,
                    self.makeWrapper(method, getattr(method, '__doc__', None)))
            self.__type = 'class'
            self.__method_name = method_name
            self.__class = klass
            
        self.__attached = 1

    def detach(self):
        """detach self from target"""
        if not self.__attached: return

        try:
            if self.__type == 'module':
                wrapper = getattr(self.__module, self.__func_name)
                setattr(self.__module, self.__func_name,
                        wrapper.func_globals['wrappee'])
            elif self.__type == 'class':
                wrapper = getattr(self.__class, self.__method_name).im_func
                setattr(self.__class, self.__method_name,
                        wrapper.func_globals['wrappee'])
        except:
            LOG('ZAK', WARNING,
                'detaching watch on %s failed (sometimes it is ok)' % \
                                                   self.getMethodPath())

        self.__attached = 0

    def getMethodPath(self):
        """return class pathname + method name"""
        return self.__method_path

    def getNCalls(self):
        """return the number of watched calls"""
        return len(self.__call_records)

    def listCalls(self):
        """return a list of the calls made"""
        return self.__call_records

        
class WatchContainer:
    """Synchronized watch container"""
    def __init__(self):
        self.__lock = Lock()
        self.__watches = {}
        self.__watch_sequence = 0

    def add(self, watch):
        self.__lock.acquire()
        try:
            self.__watch_sequence += 1
            watch.id = str(self.__watch_sequence)
            watch.attach()
            self.__watches[watch.id] = watch
        finally:
            self.__lock.release()

    def delete(self, id):
        self.__lock.acquire()
        try:
            self.__watches[id].detach()
            del self.__watches[id]
        finally:
            self.__lock.release()

    def get(self, id):
        return self.__watches[id]

    def items(self):
        return self.__watches.items()


def html_quote(s):
    """escape <, > and &"""
    return s.replace('&', '&amp;').replace('<', '&lt;').replace('>', '&gt;')

class CallRecord:
    """watch log record"""
    def __init__(self, tb, args, kwargs, thread):
        self.time = strftime("%H.%M.%S %d/%m/%Y")
        self.tb = '<br>'.join(map(html_quote, format_list(tb)[:-1]))
        self.args = ',\n'.join(map(repr, args))
        self.kwargs = repr(kwargs)
        self.thr_id = thread.getName()

    def storeRetVal(self, ret_val):
        self.ret_val = repr(ret_val)
