####################
# Zope Army Knife  #
# (c) 2003 NAUMEN  #
####################
"""
Revision information:
$Id: ZAK.py,v 1.10 2003/08/27 11:20:22 kvas Exp $

ZAK class
"""
__version__ = '$Revision: 1.10 $'[11:-2]

from posixpath import split as splitpath
from re import sub
from sys import exc_info
from traceback import format_exception_only, format_tb

from Globals import DTMLFile, InitializeClass
from Acquisition import aq_inner, aq_parent
from AccessControl import ClassSecurityInfo
from OFS.SimpleItem import SimpleItem

import ZAKContext
from ZAKContext import html_format
from Watch import Watch, WatchContainer

manage_addZAKForm = DTMLFile( 'dtml/addZAKForm', globals() )

def manage_addZAK( self, id, add=None, REQUEST=None ):
    """Create a new ZAK and add it to container"""
    container = self.Destination()

    if add:
        zak = ZAK( id )
        container._setObject( id, zak )

    if REQUEST is not None:
        REQUEST.RESPONSE.redirect( self.DestinationURL() + '/manage_main' )


class ZAK( SimpleItem ):
    """Zope Army Knife

    Zope arbitrary expression evaluator.
    """

    meta_type = 'ZAK (Zope Army Knife)'
    security = ClassSecurityInfo()

    security.declareProtected( 'Use ZAK',
                               'index_html',
                               'manage_evaluate',
                               'manage_watch',
                               'manage_showWatch',
                               'manage_configWatch',
                               'manage_showCall',
                               'listWatches',
                               'getWatch' )

    _indexHtml = DTMLFile( 'dtml/indexHtml', globals() )
    _evaluatorForm = DTMLFile( 'dtml/evaluatorForm', globals() )

    manage_watch = DTMLFile( 'dtml/watch', globals() )
    manage_showWatch = DTMLFile( 'dtml/showWatch', globals() )
    manage_showCall = DTMLFile( 'dtml/showCall', globals() )

    manage_options = ( {'label':'Evaluator', 'action':'manage_evaluate'},
                       {'label':'Watch', 'action':'manage_watch'} ) + \
                     SimpleItem.manage_options

    __watches = WatchContainer()  # class variable -- container of all watches
    

    def __init__( self, id='zak' ):
        self.id = id

    def manage_evaluate( self, expr='', persist=None, REQUEST=None ):
        """Evaluate any expression in any context"""
	path, method = splitpath( REQUEST.URL )
	use_zmi = method.startswith( 'manage_' )
	
        if expr:
            lines = []
            context = aq_parent(self)

            scope, error = evaluate( expr, context, persist, REQUEST )

            if error:
                lines += [ ''.join( format_exception_only(error[0], error[1]) ),
                           ''.join( format_tb(error[2])[1:] ),
                           '' ]

            names = scope.keys()
            names.sort()

            lines.append( 'context = ' + str(context) )
            lines += [ str(name) + ' = ' + str(scope[name]) for name in names ]

            result = html_format( '\n'.join(lines) )
        else:
            result = ''

        template = use_zmi and self._evaluatorForm or self._indexHtml

        return template( self, REQUEST, result=result, expr=expr )

    index_html = manage_evaluate
    
    def listWatches(self):
        """Return a list of all active watches"""
        return self.__watches.items()

    def manage_addWatch(self, method_name='', REQUEST=None):
        """Add new watch on named method"""
        if method_name:
            self.__watches.add(Watch(method_name))
            
        if REQUEST:
            REQUEST.response.redirect(self.absolute_url() + '/manage_watch')

    def manage_deleteWatches(self, ids=[], delete=0, REQUEST=None):
        """Delete selected watches"""
        if delete:
	        for id in ids:
	            self.__watches.delete(id)
            
        if REQUEST:
            REQUEST.response.redirect(self.absolute_url() + '/manage_watch')

    def manage_configWatch(self, watch_id=None, delete=None,
                                                clear=None, REQUEST=None):
        """Change watch configuration"""
        if not watch_id:
            if REQUEST:
                REQUEST.response.redirect(self.absolute_url() + '/manage_watch')
            return
            
        if delete:
            self.__watches.delete(watch_id)
            if REQUEST:
                REQUEST.response.redirect(self.absolute_url() + '/manage_watch')
        elif clear:
            self.getWatch(watch_id).clear()
            if REQUEST:
                REQUEST.response.redirect(self.absolute_url() + \
                                '/manage_showWatch?watch_id=' + str(watch_id))

    def getWatch(self, id):
        return self.__watches.get(str(id))

InitializeClass( ZAK )


def evaluate( expr, context=None, persist=None, REQUEST=None, **params ):
    expr  = sub( r'\r+\n?', '\n', expr.expandtabs() ) + '\n'
    error = None

    if persist:
        scope = ZAKContext.context
    else:
        scope = {}

    # NB overrides 'context' variable
    scope.update( vars( ZAKContext ) )
    scope.update( params )

    scope['REQUEST'] = REQUEST
    scope['context'] = context
    scope['app']     = context.getPhysicalRoot()

    try:
        exec expr in scope
    except:
        error = exc_info()

    for name in dir(ZAKContext) + params.keys() + ['REQUEST','context','app']:
        try: del scope[ name ]
        except KeyError: pass

    return ( scope, error )
