####################
# Zope Army Knife  #
# (c) 2003 NAUMEN  #
####################
"""
Revision information:
$Id: ZAKContext.py,v 1.4 2003/08/20 12:26:31 vpastukhov Exp $

Namespace for code evaluation
"""
__version__ = "$Revision: 1.4 $"[11:-2]

# pre-import modules (for use in expressions)
import binascii, cgi, os, re, string, sys, types

from Acquisition import aq_parent, aq_base, aq_self, aq_inner, aq_get


# persistent evaluation context
context = {}


def html_format( text ):
    text = cgi.escape( text ) \
              .replace( '  ', '&nbsp; ' ) \
              .replace( '  ', '&nbsp; ' ) \
              .replace( '\n', '<br />\n' )
    return text

def class_tree( klass, depth=None, indent=0 ):
    if indent:
        result = ''
    else:
        result = 'Inheritance tree of class %s:\n\n' % klass.__name__

    stack = [ [ klass ] ]

    while stack:
        classes = stack[-1]
        if not classes:
            stack.pop()
            continue

        level = len(stack) + indent
        klass = classes.pop(0)

        mark = klass.__bases__ and '+' or '-'
        mod  = getattr( klass, '__module__', '' )
        name = ( mod and mod + '.' ) + getattr( klass, '__name__', repr(klass) )

        result += '%s%s %s\n' % ( '  '*level, mark, name )

        if klass.__bases__ and (depth is None or level < depth):
            stack.append( list( klass.__bases__ ) )

    return result

def attr_tree( klass, attr, indent=0 ):
    if indent:
        result = ''
    else:
        result = 'Inheritance tree of attribute %s.%s:\n\n' % ( klass.__name__, attr )

    stack = [ [ klass ] ]
    seen  = 0

    while stack:
        classes = stack[-1]
        if not classes:
            stack.pop()
            continue

        level = len(stack) + indent
        klass = classes.pop(0)

        if not hasattr( klass, attr ):
            continue

        if klass.__dict__.has_key( attr ):
            mark = seen and '#' or '@'
            seen += 1
        else:
            mark = '+'

        mod  = getattr( klass, '__module__', '' )
        name = ( mod and mod + '.' ) + getattr( klass, '__name__', repr(klass) )

        result += '%s%s %s\n' % ( '  '*level, mark, name )

        if klass.__bases__:
            stack.append( list( klass.__bases__ ) )

    return result

def _getattrs( cls, attrs=None, lev=0 ):
    if attrs is None:
        attrs = {}

    for attr, val in cls.__dict__.items():
        if not attrs.has_key( attr ):
            attrs[ attr ] = []
        attrs[ attr ].append( (lev, cls, val) )

    for base in cls.__bases__:
        _getattrs( base, attrs, lev+1 )

    return attrs

_skipattrs = (
        '__doc__', '__ac_permissions__',
        '__basicnew__', '__reduce__', '__roles__',
        'inheritedAttribute',
    )

_skipregex = r'.*__roles__$'

_skipre = re.compile( _skipregex )

def attr_conflicts( klass, indent=0 ):
    if indent:
        result = ''
    else:
        result = 'Contradictions of inherited attributes in class %s:\n\n' % klass.__name__
    indent += 1

    attrs  = _getattrs( klass )
    sorted = attrs.keys()
    sorted.sort( lambda a, b: cmp( a.lower(), b.lower() ) )

    for attr in sorted:
        if attr in _skipattrs or _skipre.match( attr ):
            continue

        lst = attrs[ attr ]
        lev0, kls0, val0 = lst[0]
        count = 1
        seen  = {}

        for levi, klsi, vali in lst:
            if not ( seen.get( klsi ) or klsi is kls0 or \
                     levi > lev0 or vali is val0 ):
                count += 1
            seen[ klsi ] = 1

        if count > 1:
            result += '%s%s (%s)\n' % ( '  '*indent, attr, type(val0).__name__ )
            result += attr_tree( klass, attr, indent=indent )
            result += '\n'

    return result

def aq_tree( object, indent=0 ):
    if indent:
        result  = ''
        mark    = '+'
    else:
        result  = 'Acquisition context of object %s:\n\n' % repr( aq_inner( object ) )
        mark    = '-'
        indent += 1

    while object is not None:
        inner = object is aq_inner( object )

        if inner:
            klass = object.__class__
            mod   = getattr( klass, '__module__', '' )
            name  = ( mod and mod + '.' ) + getattr( klass, '__name__', repr(klass) )

            try:
                location = '/'.join( object.getPhysicalPath() ) or '/'
            except AttributeError:
                location = '0x%08x' % id(object)

            result += '%s%s %s at %s\n' % ( '  '*indent, mark, name, location )
            mark    = '-'

        else:
            result += aq_tree( aq_self( object ), indent )

        object  = aq_parent( object )
        indent += 1

    return result

def dtml_dict( md, indent=None ):
    stack  = []
    values = []
    if indent is not None:
        result = ''
    else:
        result = 'DTML namespace object %s:\n\n' % repr( md )
        indent = 1

    while 1:
        try: item = md._pop()
        except IndexError: break
        stack.insert( 0, item )

        try:    value = repr(item)
        except: value = '<unprintable %s object>' % type(item).__name__
        values.append( '%s* %s\n' % ( '  '*indent, value ) )

    # restore dict
    for item in stack:
        md._push( item )

    result += ''.join( values )
    return result

def find_by_oid( object, oid, stack=None ):
    if len(oid) > 8:
        oid = binascii.unhexlify( '0'*( 16-len(oid) ) + oid )

    if not hasattr( object, '__class__' ):
        return None

    if getattr( object, '_p_oid', None ) == oid:
        return ''

    if stack is None:
        stack = {}
    seen = stack.has_key

    if seen( id(object) ):
        return None
    stack[ id(object) ] = 1

    # load ghost
    getattr( object, '_p_mtime', None )
    result = None

    objdict = getattr( object, '__dict__', None )
    if objdict is not None:

        for key, value in objdict.items():
            try:
                if key.startswith('_p_') or key.startswith('_v_'):
                    continue
            except:
                pass
            if seen( id(value) ):
                continue
            result = find_by_oid( value, oid, stack )
            if result is not None:
                result = '.' + str(key) + result
                break

    getitem = getattr( object, '__getitem__', None )
    if result is None and callable( getitem ):

        items = getattr( object, 'items', None )
        if callable( items ):
            for key, value in items():
                if seen( id(value) ):
                    continue
                result = find_by_oid( value, oid, stack )
                if result is not None:
                    result = '[' + repr(key) + ']' + result
                    break

        elif hasattr( object, '__len__' ):
            try:    length = len(object)
            except: length = 0
            for idx in range( length ):
                try: item = object[ idx ]
                except: continue
                if seen( id(item) ):
                    continue
                result = find_by_oid( item, oid, stack )
                if result is not None:
                    result = '[' + repr(idx) + ']' + result
                    break

    return result
