####################
# Zope Army Knife  #
# (c) 2003 NAUMEN  #
####################
"""
Revision information:
$Id: ZAKTag.py,v 1.5 2003/08/05 09:07:29 vpastukhov Exp $

DTML ZAK tag class
"""
__version__ = '$Revision: 1.5 $'[11:-2]

from cgi import escape
from re import search, sub
from sys import exc_info
from traceback import format_exception_only, format_tb

from DocumentTemplate.DT_Util import parse_params, render_blocks

import ZAKContext
from ZAK import evaluate
from ZAKContext import html_format, dtml_dict


class ZAKTag:
    name = 'zak'
    blockContinuations = []

    def __init__( self, blocks ):
        tname, args, section = blocks[0]
        self.blocks = section.blocks

        args = parse_params( args, tag='zak', plain=1, persist=1, stack=1, request=1 )
        for name, value in args.items():
            try: args[ name ] = int( value )
            except ValueError: pass

        self.plain_text   = args.get('plain')
        self.persistent   = args.get('persist')
        self.dump_stack   = args.get('stack')
        self.dump_request = args.get('request')

    def __call__( self, md ):
        lines   = []
        context = md['this']
        REQUEST = md['REQUEST']

        expr  = render_blocks( self.blocks, md )
        match = search( r'(?m)^\s*(?=\S)', expr.expandtabs() )
        if match and match.end() > match.start():
            expr = sub( r'(?m)^\s{1,%d}' % (match.end() - match.start()), '', expr )

        scope, error = evaluate( expr, context, self.persistent, REQUEST, _=md )

        names = scope.keys()
        names.sort()

        if self.plain_text:
            if error:
                lines += [ ''.join( format_exception_only(error[0], error[1]) ),
                           ''.join( format_tb(error[2])[1:] ),
                           '' ]

            lines.append( 'context = ' + str(context) )
            lines += [ str(name) + ' = ' + str(scope[name]) for name in names ]
            lines.append( '' )

            result = '\n'.join( lines )

            if self.dump_stack:
                result += '\n' + dtml_dict(md)

            if self.dump_request:
                result += '\nREQUEST object:\n\n' + REQUEST.text()

        else:
            if error:
                lines += [ escape(line) for line in format_exception_only(error[0], error[1]) ] \
                      +  [ '' ] \
                      +  [ escape(line) for line in format_tb(error[2])[1:] ] \
                      +  [ '' ]

            lines.append( '<b>context</b>&nbsp;=&nbsp;' + escape(str(context)) )
            lines += [ '<b>' + str(name) + '</b>&nbsp;=&nbsp;' \
                       + escape(str(scope[name])) for name in names ]

            result = '<p><code>\n' + '<br/>\n'.join(lines) + '</code></p>\n'

            if self.dump_stack:
                result += '<h3>DTML namespace object:</h3>\n<p><code>\n' \
                       +  html_format( dtml_dict(md, 0) ) + '</code></p>\n'

            if self.dump_request:
                result += '<h3>REQUEST object:</h3>\n' + str(REQUEST)

        return result

    render = __call__
